@@ -1,4 +1,6 @@
 """Tasmota config validation."""
+from __future__ import annotations
+
 from typing import Any, List, TypeVar, Union
 
 import voluptuous as vol
@@ -21,7 +23,7 @@ def ensure_list(value: Union[T, List[T], None]) -> List[T]:
     return value if isinstance(value, list) else [value]
 
 
-def optional_string(value: Any) -> str:
+def optional_string(value: Any) -> str | None:
     """Coerce value to string, except for None."""
     if value is None:
         return None
